package com.yuandian.dataflow.utils;

import java.util.ArrayList;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class Utils {


     /**
     * 找到第n个substr索引点的字符串索引 
     * @param str 原始字符串
     * @param substr 需要定位的字符串
     * @param n 第几个
     * @return
     */
    public static int indexOf(String str, String substr, int n) {
        int pos = str.indexOf(substr);
        while (--n > 0 && pos != -1) {
            pos = str.indexOf(substr, pos + 1);
        }
        return pos;
    }


    /**
     * 尽可能的找到n个substr索引点的字符串索引. 如果超出就拿最大值.
     * @param str 原始字符串
     * @param substr 需要定位的字符串
     * @param n 第几个
     * @return
     */
    public static int indexOfAsPossible(String str, String substr, int n) {
        int pos = str.indexOf(substr);
        while (--n > 0 && pos != -1) {
            pos = str.indexOf(substr, pos + 1);
        }
        if(pos == -1) {
            return str.length();
        }
        return pos;
    }
 

    

    /**
     * 负载分配算法
     * @param total 总的任务数
     * @param canTasks 每个节点可以处理的任务数
     * @return 返回每个节点应该分配的任务数
     */
    public static int[]  allocationTasks(int total, int[] canTasks) {
        int[] atasks = new int[canTasks.length];
        
        int totalCans = 0;
        for(var i = 0;i < canTasks.length;i++) {
            var can = canTasks[i];
            totalCans += can;
        }

        if(total - totalCans > 0)  {
            for(var i = 0;i < canTasks.length;i++) {
                atasks[i] = canTasks[i];
            }
            return atasks;
        }

        // 剩下来的平均值
        int remainAvg = (totalCans - total) / canTasks.length;
         
        
        for(var i = 0;i < canTasks.length;i++) {
            var can = canTasks[i];
            if(remainAvg <=  can) {
                atasks[i] = can - remainAvg;
            }
        }
         
    
        return atasks;
    }

     
}
